# NOTE: Derived from blib/lib/ClearCase/Wrapper/MGi.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package ClearCase::Wrapper::MGi;

#line 4058 "blib/lib/ClearCase/Wrapper/MGi.pm (autosplit into blib/lib/auto/ClearCase/Wrapper/MGi/synctree.al)"
sub synctree {
  use strict;
  use warnings;
  use ClearCase::SyncTree 0.60; #warning: sort interpreted as function
  use Benchmark;
  use Cwd;
  my %opt;
  GetOptions(\%opt, qw(from=s summary label=s comment=s force rollback));
  die Msg("-force and -rollback are mutually exclusive")
    if $opt{force} and $opt{rollback};
  Assert(@ARGV > 1);		# die with usage msg if untrue
  shift @ARGV;
  my @argv = ();
  for (@ARGV) {
    $_ = readlink if -l && defined readlink;
    push @argv, MSWIN ? glob($_) : $_;
  }
  ClearCase::Argv->inpathnorm(0);
  if ($opt{summary}) {
    $Benchstart = new Benchmark;
    ClearCase::Argv->summary;	# start keeping stats
    END {
      if ($Benchstart) {
	# print out the stats we kept
	print STDERR ClearCase::Argv->summary;
	# show timing data
	my $timing = timestr(timediff(new Benchmark, $Benchstart));
	print "Elapsed time: $timing\n";
      }
    }
  }
  my $sync = ClearCase::SyncTree->new;
  if (@argv == 1 and (-d $argv[0] or ! -e $argv[0])) {
    $opt{dbase} = $sync->dstbase($argv[0]);
    @argv = ();
  } else {
    $opt{dbase} = $sync->dstbase(dirname($argv[0]));
  }
  die Msg('E', "no such directory $opt{from}") unless -d $opt{from};
  $opt{sbase} = Cwd::realpath($opt{from});
  $opt{sbase} =~ s%\\%/%g if MSWIN;
  ClearCase::Argv->quiet(1) if $opt{quiet};
  if ($opt{label}) {
    my $ct = $sync->clone_ct({autofail=>0, stderr=>0});
    my $dvob = $ct->des(['-s'], "vob:$opt{dbase}")->qx;
    my $lbtype = "lbtype:$opt{label}\@$dvob";
    $sync->lblver($opt{label}) if $opt{vreuse} && $ct->des(['-s'], $lbtype)->qx;
    my ($inclb) = grep s/-> (lbtype:.*)$/$1/,
      $ct->des([qw(-s -ahl EqInc)], $lbtype)->qx;
    if ($inclb) {
      die "$prog: Error: incremental label types must be unlocked\n"
	if $ct->lslock(['-s'], $lbtype, $inclb)->qx;
      $inclb =~ s/^lbtype:(.*)@.*$/$1/;
      $sync->inclb($inclb);
    }
  }
  {
    my @src;
    if (@argv) {
      my @abort;
      for my $arg (@argv) {
	if (-r $arg) {
	  my $real = Cwd::realpath($arg);
	  $real = $sync->normalize($real);
	  if ($real =~ s/^\Q$opt{dbase}\E/$opt{sbase}/ and -r $real) {
	    push @src, $real;
	    next;
	  }
	}
	push @abort, $arg;
      }
      die Msg('E', "argument" . (@abort > 1? 's' : '') . " not found:\n  "
		. join("\n  ", @abort)) if @abort;
    } else {
      push @src, $opt{sbase};
    }
    local $SIG{__WARN__} = sub { die Msg('E', @_) };
    my %cfg;
    $cfg{wanted} = \&_Wanted;
    find(\%cfg, $_) for @src;
  }
  $sync->reuse(1);
  $sync->vreuse(1) if $opt{label};
  $sync->dstcheck;
  my $rc = 0;
  $sync->err_handler(sub {exit 2}) unless $opt{cleanup} or $opt{force};
  $sync->err_handler(\$rc) if $opt{force};
  $opt{comment} = 'imported with "ct synctree"' unless $opt{comment};
  $sync->comment($opt{comment});
  $sync->srcbase($opt{sbase});
  $sync->srcmap(%Xfer);
  $sync->remove(1) unless @argv;
  $sync->rellinks(1);
  $sync->analyze;
  $sync->rmdirlinks;
  $sync->add;
  $sync->modify;
  $sync->subtract unless @argv;
  $sync->label($opt{label}) if $opt{label};
  exit $rc unless $sync->get_addhash || $sync->get_modhash
		                     || $sync->get_sublist || $sync->_lsco;
  $sync->err_handler(\$rc);
  if ($ENV{FSCBROKER}) {
    require ClearCase::FixSrcCont; #optional fix of source container
    my $ct = $sync->clone_ct({autofail=>0});
    for ($ct->lsco([qw(-cview -me -a -fmt), '%PVn %[hlink:Merge]p\n'],
		   $opt{dbase})->qx) {
      next unless m%^\S+/0 "Merge\@.*?" <- "(.*?)"%;
      ClearCase::FixSrcCont::add2fix($1);
    }
    ClearCase::FixSrcCont::runfix();
  }
  $sync->checkin;
  exit $rc;
}

1;
# end of ClearCase::Wrapper::MGi::synctree
